import os.path

from .base import Model, Collection, DATA_DIRECTORY

PRECINCT_CSV_FILENAME = os.path.join(DATA_DIRECTORY, 'chicago_precinct_census_tract_crosswalk.csv')

class Precinct(Model):
    fields = [
        'number',
        'ward',
        'full_name',
        'census_tract_geoid'
    ]

    def __str__(self):
        return self.full_name

    def __repr__(self):
        return "Precinct(ward='{p.ward}', number='{p.number}', full_name='{p.full_name}', census_tract_geoid='{p.census_tract_geoid}')".format(
            p=self)


class PrecinctCollection(Collection):
    model = Precinct

    def __init__(self):
        self._by_full_name = {}
        super(PrecinctCollection, self).__init__()

    def add_item(self, item):
        super(PrecinctCollection, self).add_item(item)
        self._by_full_name[item.full_name] = item

    def transform_row(self, row):
        return {
            'number': row['precinct_number'],
            'ward': row['precinct_ward'],
            'full_name': row['precinct_full_name'],
            'census_tract_geoid': row['tract_geoid']
        }

    def get_by_full_name(self, full_name):
        return self._by_full_name[str(full_name)]

    def default_sort(self):
        self._items = sorted(self._items, key=lambda p: int(p.full_name))
        return self


PRECINCTS = PrecinctCollection().from_csv(PRECINCT_CSV_FILENAME)


def get_precincts_from_tract_geoid(geoid):
    matching_precincts = []
    for precinct in PRECINCTS:
        if precinct.census_tract_geoid == str(geoid):
            matching_precincts.append(precinct)
    return matching_precincts



from .community_areas import COMMUNITY_AREAS
from .neighborhoods import NEIGHBORHOODS
from .precincts import PRECINCTS, get_precincts_from_tract_geoid
from .tracts import TRACTS, get_tract_from_ward_and_precinct, get_tract_from_precinct_id
from .illinois.counties import COUNTIES
from .cook_suburbs.precincts import COOK_SUBURBAN_PRECINCTS, COOK_SUBURBAN_CROSSWALK, get_suburban_cook_precincts_from_tract_geoid, get_suburban_cook_tract_from_precinct_number
from .zip_codes import ZIP_CODES



import os.path

import six

from .base import Model, Collection, DATA_DIRECTORY


ZIP_CODE_GEOJSON_PATH = os.path.join(DATA_DIRECTORY, 'chicago_zip_codes.geojson')

class ZipCode(Model):
    fields = [
        'zip',
    ]


class ZipCodeCollection(Collection):
    model = ZipCode

    def __init__(self):
        self._by_zip = {}
        super(ZipCodeCollection, self).__init__()

    def add_item(self, item):
        super(ZipCodeCollection, self).add_item(item)
        self._by_zip[item.zip] = item

    def get_by_zip(self, zip_code):
        return self._by_zip[six.text_type(zip_code)]

    def default_sort(self):
        self._items = sorted(self._items, key=lambda p: p.zip)
        return self

    def is_chicago(self, zip_code):
        return six.text_type(zip_code) in self._by_zip


ZIP_CODES = ZipCodeCollection().from_geojson(ZIP_CODE_GEOJSON_PATH)



import os.path

from .base import Model, Collection, DATA_DIRECTORY

TRACT_CSV_FILENAME = os.path.join(DATA_DIRECTORY, 'chicago_precinct_census_tract_crosswalk.csv')

class Tract(Model):
    fields = [
        'commarea_num',
        'countyfp',
        'geoid',
        'name',
        'statefp',
        'precinct_full_name'
    ]

    def __str__(self):
        return self.geoid

    def __repr__(self):
        return "Tract(commarea_num='{t.commarea_num}',countyfp='{t.countyfp}',geoid='{t.geoid}',name='{t.name}',statefp='{t.statefp}',precinct_full_name='{t.precinct_full_name}')".format(
            t=self)


class TractCollection(Collection):
    model = Tract

    def __init__(self):
        self._by_geoid = {}
        super(TractCollection, self).__init__()

    def add_item(self, item):
        super(TractCollection, self).add_item(item)
        self._by_geoid[item.geoid] = item

    def transform_row(self, row):
        return {
            'commarea_num': row['tract_commarea_num'],
            'countyfp': row['tract_countyfp'],
            'geoid': row['tract_geoid'],
            'name': row['tract_name'],
            'statefp': row['tract_statefp'],
            'precinct_full_name': row['precinct_full_name']
        }

    def get_by_geoid(self, geoid):
        return self._by_geoid[str(geoid)]

    def default_sort(self):
        self._items = sorted(self._items, key=lambda p: int(p.geoid))
        return self


TRACTS = TractCollection().from_csv(TRACT_CSV_FILENAME)


def get_tract_from_ward_and_precinct(ward, precinct):
    precinct_id = '%s%s' % (str(ward).zfill(2), str(precinct).zfill(3))
    return get_tract_from_precinct_id(precinct_id)


def get_tract_from_precinct_id(precinct_id):
    for tract in TRACTS:
        if tract.precinct_full_name == str(precinct_id):
            return tract
    return None



import os.path
import sys
sys.path.append(os.path.join(os.path.dirname(os.path.realpath(__file__)), os.pardir))

from base import Model, Collection, DATA_DIRECTORY

COUNTY_CSV_FILENAME = os.path.join(DATA_DIRECTORY, 'county_fips.csv')

class County(Model):
    fields = [
        'state',
        'statefp',
        'countyfp',
        'countyname'
    ]

    def __str__(self):
        return self.countyname

    def __repr__(self):
        return "County(state='{c.state}', statefp='{c.statefp}', countyfp='{c.countyfp}', countyname='{c.countyname}')".format(
            c=self)


class CountyCollection(Collection):
    model = County

    def __init__(self):
        self._by_fips = {}
        self._by_name = {}
        super(CountyCollection, self).__init__()

    def add_item(self, item):
        super(CountyCollection, self).add_item(item)
        self._by_fips[item.countyfp] = item
        self._by_name[item.countyname.lower()] = item

    def transform_row(self, row):
        return {
            'state': row['State'],
            'statefp': str(row['StateFP']),
            'countyfp': str(row['CountyFP']),
            'countyname': row['County'].replace(' County', '')
        }

    def get_by_name(self, name):
        return self._by_name.get(str(name).lower(), None)

    def get_by_fips(self, fips):
        return self._by_fips.get(str(fips), None)

    def default_sort(self):
        self._items = sorted(self._items, key=lambda c: c.countyname)
        return self


COUNTIES = CountyCollection().from_csv(COUNTY_CSV_FILENAME)



import os.path
import sys
sys.path.append(os.path.join(os.path.dirname(os.path.realpath(__file__)), os.pardir))

from base import Model, Collection, DATA_DIRECTORY
from csv import DictReader

COOK_SUBURBAN_PRECINCT_CSV_FILENAME = os.path.join(
    DATA_DIRECTORY, 'cook_suburban_precincts_as_of_2016.csv')
COOK_SUBURBAN_PRECINCT_TRACT_CROSSWALK_CSV_FILENAME = os.path.join(
    DATA_DIRECTORY, 'suburban_cook_precinct_census_tract_crosswalk.csv')

class CookSuburbanPrecinct(Model):
    fields = [
        'town',
        'precinctid',
        'objectid'
    ]

    def __str__(self):
        return self.precinctid

    def __repr__(self):
        return "CookSuburbanPrecinct(town='{pc.town}', precinctid='{pc.precinctid}', objectid='{pc.objectid}'')".format(
            pc=self)


class CookSuburbanPrecinctCollection(Collection):
    model = CookSuburbanPrecinct

    def __init__(self):
        self._by_precinct_id = {}
        self._by_town_name = {}
        self._by_object_id = {}
        super(CookSuburbanPrecinctCollection, self).__init__()

    def add_item(self, item):
        super(CookSuburbanPrecinctCollection, self).add_item(item)
        self._by_precinct_id[item.precinctid] = item
        self._by_object_id[item.objectid] = item
        if item.town.lower() not in self._by_town_name:
            self._by_town_name[item.town.lower()] = []
        self._by_town_name[item.town.lower()].append(item)

    def transform_row(self, row):
        return {
            'town': row['name'],
            'precinctid': str(row['idpct']),
            'objectid': row['objectid']
        }

    def get_by_town_name(self, name):
        return self._by_town_name.get(str(name).lower(), None)

    def get_by_precinct_id(self, precinctid):
        return self._by_precinct_id.get(str(precinctid), None)

    def get_by_object_id(self, object_id):
        return self._by_object_id.get(str(object_id), None)

    def default_sort(self):
        self._items = sorted(self._items, key=lambda pc: int(pc.precinctid))
        return self


COOK_SUBURBAN_PRECINCTS = CookSuburbanPrecinctCollection().from_csv(
    COOK_SUBURBAN_PRECINCT_CSV_FILENAME)

# HACK - this seems duplicative with Chicago precinct crosswalk
COOK_SUBURBAN_CROSSWALK = []
with open(COOK_SUBURBAN_PRECINCT_TRACT_CROSSWALK_CSV_FILENAME) as fh:
    reader = DictReader(fh)
    for row in reader:
        COOK_SUBURBAN_CROSSWALK.append(row)

def get_suburban_cook_precincts_from_tract_geoid(geoid, precinct_key='precinct_objectid'):
    precinct_ids = []
    for row in COOK_SUBURBAN_CROSSWALK:
        if row['tract_geoid'] == str(geoid):
            precinct_ids.append(row.get(precinct_key, None))
    return precinct_ids

def get_suburban_cook_tract_from_precinct_number(precinct_id, precinct_key='precinct_objectid'):
    for row in COOK_SUBURBAN_CROSSWALK:
        if row.get(precinct_key, None) == str(precinct_id):
            return row['tract_geoid']
    return None